#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['wily', 'wily.archivers', 'wily.commands', 'wily.helper', 'wily.operators']

package_data = \
{'': ['*'],
 'wily': ['locales/*',
          'locales/de/*',
          'locales/de/LC_MESSAGES/*',
          'locales/en/*',
          'locales/en/LC_MESSAGES/*',
          'locales/en_AU/*',
          'locales/en_AU/LC_MESSAGES/*',
          'locales/ja/*',
          'locales/ja/LC_MESSAGES/*',
          'templates/*',
          'templates/css/*']}

package_dir = \
{'': 'src'}

install_requires = \
['gitpython>=3.0.0,<4.0.0',
 'radon>=4.0.0,<4.1.0',
 'click>=7.0,<8.0',
 'nbformat>=4.4.0,<4.5.0',
 'colorlog>=4.0.0,<5.0.0',
 'tabulate>=0.8.2,<1.0.0',
 'plotly>=4.0.0,<5.0.0',
 'progress==1.4']

extras_require = \
{":python_version == '3.6'": ['dataclasses']}

entry_points = \
{'console_scripts': ['wily = wily.__main__:cli']}

setup(name='wily',
      version='1.19.0',
      description='A Python application for tracking, reporting on timing and complexity in tests and applications.',
      author='Anthony Shaw',
      author_email='anthonyshaw@apache.org',
      url='https://github.com/tonybaloney/wily',
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
